@@ -25,9 +25,8 @@ LOG = logging.getLogger(__name__)
 CONFIG_GENERAL = configuration.Configuration.instance().config
 CONFIG = CONFIG_GENERAL["hmm"]
 
+
 ################################### HMM class ############################
-
-
 class HMM:
 
     """ Sequence profile hidden Markov models (HMMs) are used to model genomic
@@ -113,21 +112,99 @@ class HMM:
                 self.id,
                 self.l_effective)
             hmm += "Most likely motif: {}".format(tmp)
-        except:
+        except (AttributeError, TypeError):
             hmm = "<HMM instance>"
             LOG.warning("Could not create string of HMM instance.")
 
         return hmm
 
     # Use functions defined in other methods as class functions.
-    # Static function (declare as @static ?):
-    read = hmm_io.read
+    # Static function
+    @staticmethod
+    def read(hmm_filename, id=None):
+        """Read HMM file in HMMER3 format.
+
+        HMMER3 file format is described in detail in
+        ftp://selab.janelia.org/pub/software/hmmer3/3.0/Userguide.pdf,
+        section 8.
+
+        The general file format is::
+
+            HMMER3/b [3.0b2 | June 2009]
+            NAME fn3
+            ACC PF00041.12
+            DESC Fibronectin type III domain
+            LENG 86
+            ALPH amino
+            (...)
+
+            HMM          A       C       D       E       F       G       H       I    (...)    Y
+                        m->m    m->i    m->d    i->m    i->i    d->m    d->d
+              COMPO   2.70271 4.89246 3.02314 2.64362 3.59817 2.82566 3.74147 3.08574 (...) 3.22607
+                      2.68618 4.42225 2.77519 2.73123 3.46354 2.40513 3.72494 3.29354 (...) 3.61503
+                      0.00338 6.08833 6.81068 0.61958 0.77255 0.00000       *
+              1       3.16986 5.21447 4.52134 3.29953 4.34285 4.18764 4.30886 3.35801 (...) 3.93889 1 - -
+                      2.68629 4.42236 2.77530 2.73088 3.46365 2.40512 3.72505 3.29365 (...) 3.61514
+                      0.09796 2.38361 6.81068 0.10064 2.34607 0.48576 0.95510
+              2       2.70230 5.97353 2.24744 2.62947 5.31433 2.60356 4.43584 4.79731 (...) 4.25623 3 - -
+                      2.68618 4.42225 2.77519 2.73123 3.46354 2.40513 3.72494 3.29354 (...) 3.61503
+                      0.00338 6.08833 6.81068 0.61958 0.77255 0.48576 0.95510
+              (...)
+              86      3.03720 5.94099 3.75455 2.96917 5.26587 2.91682 3.66571 4.11840 (...) 4.99111 121 - E
+                      2.68618 4.42225 2.77519 2.73123 3.46354 2.40513 3.72494 3.29354 (...) 3.61503
+                      0.00227 6.08723       * 0.61958 0.77255 0.00000       *
+            //
+
+        .. important:: The probabilities in this format are stored as negative
+            natural log probabilities, e.g. -ln(0.25) = 1.38629. The special case
+            of 0 probability is stored as ``*`` which in fact means -∞ (minus
+            infinity).
+
+        Args:
+            hmm_filename (str): Path to the file with model data in the HMMER3
+                file format.
+            id (str, optional): The identifier for the model to be returned.
+                E.g. for Pfam the ``id`` can look like this: 'PF00560'.
+                If defined, the function returns only the HMM with an identifier
+                that matches the provided id. If a model in the file does not have
+                an identifier to check against, it is skipped.
+
+        Returns:
+            dict: A dictionary of parameters required to initialise the HMM
+            including the id, alphabet, emission probabilities and transition
+            probabilities.
+
+            Output format::
+
+                {
+                    'id': 'PF08261.7',
+                    'letters': ['A', 'C', 'D', ..., 'Y'],
+                    'COMPO':
+                        {'insertion_emissions': [2.68618, 4.42225, ..., 3.61503],
+                          'emissions': [2.28205, 5.14899, ..., 1.92022],
+                          'transitions': [0.01467, 4.62483, ..., -inf]},
+                    '1':
+                        {'insertion_emissions': [2.68618, 4.42225, ..., 3.61503],
+                         'emissions': [1.00089, 4.54999, ..., 5.23581],
+                         'transitions': [0.01467, 4.62483, ..., 0.95510]},
+                    ...
+                    '8':
+                        {'insertion_emissions': [2.68618, 4.42225, ..., 3.61503],
+                         'emissions': [4.12723, 5.39816, ..., 4.58094],
+                         'transitions': [0.00990, 4.62006, ..., -inf]}
+                }
+
+        See Also:
+            hmm_io.read()
+
+        """
+        return hmm_io.read(hmm_filename, id)
 
     # Function that take the HMM instance as argument:
     def viterbi(self, *args):
         return hmm_viterbi.viterbi(self, *args)
 
-    def __init__(self, hmmer_probabilities):
+    def __init__(self, hmmer_probabilities, sequence_type="AA"):
         """ HMM class __init_ module.
 
         __init__ takes HMM parameters (including the alphabet, emission
@@ -140,9 +217,10 @@ class HMM:
         self.hmmer = hmmer_probabilities
         self.id = hmmer_probabilities['id']
         self.alphabet = self.hmmer['letters']
+        self.sequence_type = sequence_type
 
-        self.l_effective = max([int(key) for key \
-                               in hmmer_probabilities.keys() if key.isdigit()])
+        self.l_effective = max([int(key) for key
+                                in hmmer_probabilities.keys() if key.isdigit()])
 
         # Initialise all HMM states to default value (e.g. transition to or
         # from terminal state all have the same cost 0).
@@ -167,8 +245,7 @@ class HMM:
                 i_emission_probabilities)
         for i_state in (self.insertion_states + self.terminal_states):
             i_emission_probabilities = \
-                self.hmmer[d_translate_states[i_state]] \
-                ['insertion_emissions'][:len(self.alphabet)]
+                self.hmmer[d_translate_states[i_state]]['insertion_emissions'][:len(self.alphabet)]
             self.set_emission_probability_hmmer3(
                 i_state,
                 i_emission_probabilities)
@@ -216,14 +293,15 @@ class HMM:
         self.states = [self.terminal_states[
             0]] + self.match_states + self.insertion_states + [self.terminal_states[1]]
         self.p_t = {i_state: {i: j for i, j in self.p_t[
-            i_state].items() if not i in self.deletion_states} for i_state in self.states}
+            i_state].items() if i not in self.deletion_states} for i_state in self.states}
         self.p_0 = {
             i: j for i,
             j in self.p_0.items() if i not in self.deletion_states}
 
         del self.deletion_states
 
-    def create(input_format, file=None, repeat=None):
+    @staticmethod
+    def create(input_format, file=None, repeat=None, **kwargs):
         """ Creates a HMM instance from 2 possible input formats.
 
         A `HMM` instance is created from one of the two possible inputs:
@@ -259,6 +337,7 @@ class HMM:
             if not os.path.exists(file):
                 raise Exception('HMMER3 file does not exist.')
             hmmer_probabilities = next(HMM.read(file))
+            sequence_type = "AA"
         elif input_format == 'pickle':
             with open(file, 'rb') as fh:
                 return pickle.load(fh)
@@ -266,13 +345,19 @@ class HMM:
             if not isinstance(repeat, Repeat):
                 raise Exception('The repeat value is not a valid instance of '
                                 'the Repeat class.')
-            hmmer_probabilities = HMM.create_from_repeat(repeat)
+            if 'hmmbuild' in kwargs:
+                hmmer_probabilities = HMM.create_from_repeat(repeat, **kwargs['hmmbuild'])
+            else:
+                hmmer_probabilities = HMM.create_from_repeat(repeat)
+            sequence_type = repeat.sequence_type
+
         else:
             raise Exception("Unknown input format: {}.".format(input_format))
 
         LOG.debug(hmmer_probabilities)
-        return HMM(hmmer_probabilities)
+        return HMM(hmmer_probabilities, sequence_type)
 
+    @staticmethod
     def create_from_repeat(tandem_repeat, hmm_copy_path=None,
                            hmm_copy_id=None):
         """ Get HMM parameters (including the alphabet, emission probabilities
@@ -312,8 +397,16 @@ class HMM:
         # O repeat_io.save_repeat_stockholm(tandem_repeat.msaD, stockholm_file)
         tandem_repeat.write(file=stockholm_file, file_format="stockholm")
 
+        # Define hmmbuild sequence type flag.
+        if tandem_repeat.sequence_type == "AA":
+            sequence_type_flag = "--amino"
+        elif tandem_repeat.sequence_type == "RNA":
+            sequence_type_flag = "--rna"
+        else:
+            sequence_type_flag = "--dna"
+
         # Run HMMbuild to build a HMM model, and read model
-        p = subprocess.Popen([CONFIG["hmmbuild"], "--amino", tmp_id + ".hmm",
+        p = subprocess.Popen([CONFIG["hmmbuild"], sequence_type_flag, tmp_id + ".hmm",
                               tmp_id + ".sto"],
                              stdout=subprocess.PIPE, stderr=None, cwd=tmp_dir)
         p.wait()
@@ -361,7 +454,7 @@ class HMM:
     def hmm_example(self):
         """ .. todo:: Is this method needed?
         """
-        #states = ["N", "B", "M1", "M2", "M3", "E", "C"]
+        # states = ["N", "B", "M1", "M2", "M3", "E", "C"]
         self.states = ["N", "M1", "M2", "C"]
 
         # Initialisation
@@ -426,8 +519,7 @@ class HMM:
         for i in range(l_effective):
             self.p_t["N"]["M{0}".format(i % l_effective + 1)] = 0
             self.p_t["M{0}".format(i % l_effective + 1)]["C"] = 0
-            self.p_t["M{0}".format(i % l_effective + 1)] \
-                    ["M{0}".format((i + 1) % l_effective + 1)] = 0
+            self.p_t["M{0}".format(i % l_effective + 1)]["M{0}".format((i + 1) % l_effective + 1)] = 0
         self.p_t["N"]["N"] = 0
         self.p_t["C"]["C"] = 0
 
@@ -442,9 +534,10 @@ class HMM:
         In HMMER3 data, emission probabilities are -ln(p).
 
         It is e.g.::
+
             self.alphabet =
-            ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P',
-             'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
+            ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q',
+             'R', 'S', 'T', 'V', 'W', 'Y']
 
         Return log10(p), that is convert between the two. Conversion:
         p_Local = - p_HMM * log10(e)
@@ -462,8 +555,8 @@ class HMM:
         l_emission_probabilities = \
             [-(i) * np.log10(np.exp(1)) for i in l_emission_probabilities]
         self.p_e[state] = \
-            {iL: iEP for iL, iEP in \
-            zip(self.alphabet, l_emission_probabilities)}
+            {iL: iEP for iL, iEP in
+             zip(self.alphabet, l_emission_probabilities)}
 
     def set_circle_transition_probability_hmmer3(
             self,
@@ -567,10 +660,10 @@ class HMM:
             l_transition_probabilities (list of float): A list of transition
                 probabilities as found in a single row for a certain state in
                 HMMER3 files. E.g. ['0.00021', '8.85487', '9.57722', '0.61958',
-                 '0.77255', '0.48576', '0.95510']
+                '0.77255', '0.48576', '0.95510']
         """
 
-        l_transition_probabilities = [-(i) * np.log10(np.exp(1)) for i \
+        l_transition_probabilities = [-(i) * np.log10(np.exp(1)) for i
                                       in l_transition_probabilities]
 
         # Match state state_index -> Match state state_index + 1
@@ -657,6 +750,7 @@ def hmmer3_emission_probabilities(hmmer_probabilities, letters, l_match):
     Conversion: p_Local = - p_HMM * log10(e)
 
     Parameters (e.g.)::
+
         letters = ['A', 'C', 'E', 'D', 'G', 'F', 'I', 'H', 'K', 'M', 'L', 'N',
                    'Q', 'P', 'S', 'R', 'T', 'W', 'V', 'Y']
         l_match = ['M'+str(i) for i in range(24)]
