@@ -14,7 +14,7 @@ import numpy as np
 import pickle
 import re
 
-from tral.repeat import repeat_score, repeat_pvalue, repeat_io
+from tral.repeat import repeat_score, repeat_pvalue, repeat_io, repeat_align
 from tral import configuration
 
 LOG = logging.getLogger(__name__)
@@ -114,14 +114,14 @@ class Repeat:
                 return pickle.load(fh)
         elif input_format == 'fasta':
             l_repeat = repeat_io.read_fasta(file, **kwargs)
-            return [Repeat(msa=i_msa, sequence_type=i_sequence_type) \
+            return [Repeat(msa=i_msa, sequence_type=i_sequence_type)
                     for i_msa, i_sequence_type in l_repeat]
         elif input_format == 'simulate':
-            l_repeat = repeat_io.random_sequence(return_type='repeat')
-            return [Repeat(msa=i_msa, sequence_type=i_sequence_type) \
+            l_repeat = repeat_io.random_sequence(return_type='repeat', **kwargs)
+            return [Repeat(msa=i_msa, sequence_type=i_sequence_type)
                     for i_msa, i_sequence_type in l_repeat]
         else:
-            raise Exception('file_format is unknown: {}.'.format(file_format))
+            raise Exception('input_format is unknown: {}.'.format(input_format))
 
     def write(self, file, file_format):
         """ Write tandem repeat to file.
@@ -190,10 +190,10 @@ class Repeat:
         self.msa = [repeat_unit.upper().replace("_", "-").replace("*", "-")
                     for repeat_unit in msa]
 
-        self.msa_standard_aa = [standardize(i) for i in self.msa]
-
         self.sequence_type = sequence_type
 
+        self.msa_standard_aa = [standardize(i, self.sequence_type) for i in self.msa]
+
         self.n = len(self.msa)
         self.l_msa = max(map(len, self.msa))
 
@@ -314,8 +314,8 @@ class Repeat:
             if 'phylo_gap01' in scoreslist:
                 self.d_divergence['phylo_gap01'], self.d_score['phylo_gap01'] = \
                     repeat_score.phylo_star_topology_local(self,
-                                                         gaps='row_wise',
-                                                         indel_rate_per_site=0.01)
+                                                           gaps='row_wise',
+                                                           indel_rate_per_site=0.01)
             if 'phylo_gap01_ignore_trailing_gaps' in scoreslist:
                 self.d_divergence['phylo_gap01_ignore_trailing_gaps'], \
                     self.d_score['phylo_gap01_ignore_trailing_gaps'] = \
@@ -326,7 +326,8 @@ class Repeat:
             if 'phylo_gap01_ignore_trailing_gaps_and_coherent_deletions' in scoreslist:
                 self.d_divergence['phylo_gap01_ignore_trailing_gaps_and_coherent_deletions'], \
                     self.d_score['phylo_gap01_ignore_trailing_gaps_and_coherent_deletions'] = \
-                    repeat_score.phylo_star_topology_local(self, gaps='ignore_trailing_gaps_and_coherent_deletions', indel_rate_per_site=0.01)
+                    repeat_score.phylo_star_topology_local(
+                        self, gaps='ignore_trailing_gaps_and_coherent_deletions', indel_rate_per_site=0.01)
             if 'phylo_gap001' in scoreslist:
                 self.d_divergence['phylo_gap001'], self.d_score['phylo_gap001'] = repeat_score.phylo_star_topology_local(
                     self, gaps='row_wise', indel_rate_per_site=0.001)
@@ -353,6 +354,9 @@ class Repeat:
             ["phylo_gap01"]) for which p-Values are calculated on the
             Repeat instance `self`
         """
+        if type(scoreslist) != list:
+            raise ValueError("Please make sure that scoreslist is a list. "
+                             "You can make this sure by a comma at the end of your Config-File.")
 
         if not hasattr(self, 'd_pvalue'):
             self.d_pvalue = defaultdict(int)
@@ -437,19 +441,19 @@ class Repeat:
         self.msaTD = [column for column in self.msaT
                       if (2 * column.count('-') < len(column))
                       ]
-        self.insertions = re.compile("-+").findall(''.join([
+        self.insertions = re.compile(r"-+").findall(''.join([
             'p' if (2 * column.count('-') < len(column)) else '-'
             for column in self.msaT
         ])
         )
 
-        self.msaTD_standard_aa = [standardize(i) for i in self.msaTD]
+        self.msaTD_standard_aa = [standardize(i, self.sequence_type) for i in self.msaTD]
 
         self.msaD = ["".join(c) for c in zip(*self.msaTD)]
 
         self.l_effective = len(self.msaTD)
         textD = " ".join(self.msaD)
-        self.textD_standard_aa = standardize(textD)
+        self.textD_standard_aa = standardize(textD, self.sequence_type)
         # totD is used in repeat_score
         self.totD = len(textD) - textD.count('-') - self.n + 1
         if self.l_effective != 0:
@@ -484,7 +488,7 @@ class Repeat:
         # insertions in the last column
         # and the first column as associated (= of the same origin)
         # Lastly, detect the length of all insertions.
-        insertions = re.compile("-+").findall(2 * (''.join([
+        insertions = re.compile(r"-+").findall(2 * (''.join([
             'p' if (2 * column.count('-') < len(column)) else '-'
             for column in self.msaT
         ])))
@@ -497,7 +501,7 @@ class Repeat:
         # 2. Detect deletions
         # CHECK this lines. you used msaTD before, but swapped to msaD
         deletions = [(m.start() % self.l_msa, len(m.group()))
-                     for m in re.finditer(re.compile("-+"), "".join(self.msaD))]
+                     for m in re.finditer(re.compile(r"-+"), "".join(self.msaD))]
         self.deletions['row_wise'] = [i[1] for i in deletions]
         self.gaps['row_wise'] = self.insertions + self.deletions['row_wise']
 
@@ -523,8 +527,8 @@ class Repeat:
 
         # CHECK this lines. you used msaTD before, but swapped to msaD
         # msaTD = ["".join(c) for c in zip(*msaD)]
-        deletions = [(m.start() % self.l_msa, len(m.group())) \
-                     for m in re.finditer(re.compile("-+"), "".join(msaD))]
+        deletions = [(m.start() % self.l_msa, len(m.group()))
+                     for m in re.finditer(re.compile(r"-+"), "".join(msaD))]
         self.deletions['ignore_trailing_gaps'] = [i[1] for i in deletions]
         self.gaps['ignore_trailing_gaps'] = self.insertions + \
             self.deletions['ignore_trailing_gaps']
@@ -623,14 +627,14 @@ class Repeat:
                             begin +
                             length)]
                     insertion_lengths[iL].extend(
-                        [len(iI) for iI in re.findall('\w+', insertion)])
+                        [len(iI) for iI in re.findall(r'\w+', insertion)])
 
                 # In case the insertion blocks bridges over then end of the
                 # repeat unit:
                 if begin > end:
                     insertion = repeat_sequence[:end + 1]
                     insertion_lengths[iL].extend(
-                        [len(iI) for iI in re.findall('\w+', insertion)])
+                        [len(iI) for iI in re.findall(r'\w+', insertion)])
 
         LOG.debug(
             "The insertion_lengths of the current TR are: {0}".format(insertion_lengths))
@@ -641,9 +645,9 @@ class Repeat:
         deletions_all = [(m.start(), len(m.group()))
                          for m in re.finditer('-+', ''.join(self.msaD))]
         # Group the deletions with respect to the column in which they start:
-        deletion_lengths = {iL: \
-            [iD[1] for iD in deletions_all if iD[0] % self.l_effective == iL] \
-            for iL in range(self.l_effective)}
+        deletion_lengths = {iL:
+                            [iD[1] for iD in deletions_all if iD[0] % self.l_effective == iL]
+                            for iL in range(self.l_effective)}
         LOG.debug(
             "The deletion_lengths of the current TR are: {0}".format(deletion_lengths))
 
@@ -671,13 +675,27 @@ class Repeat:
                     count += 1
             self.msa_original.append(unit_original)
 
+    def realign_TR(self, realignment='proPIP', rate_distribution=CONFIG['castor_parameter']['rate_distribution']):
+        """ Realign multiple sequence alignment of tandem repeats.
+            Update of tandem repeat MSA.
+
+        Args:
+            self (Repeat instance)
+            realignment (str): either "proPIP" or "mafft"
+            rate_distribution (str): either "constant" or "gamma" (per default value from REPEAT_CONFIG['castor_parameter']['rate_distribution'])
+        """
+        self.msa = repeat_align.realign_repeat(self.msa,
+                                               realignment,
+                                               sequence_type=self.sequence_type,
+                                               rate_distribution=rate_distribution)
+
 # Standardize MSA #############################################################
 
 
-def standardize(blob):
+def standardize(blob, sequence_type):
 
-    for original, replacement in CONFIG_GENERAL[
-            'ambiguous_amino_acid'].items():
+    for original, replacement in CONFIG_GENERAL[sequence_type][
+            'ambiguous_chars'].items():
         blob = blob.replace(original, replacement[0])
     return blob
 
