@@ -9,14 +9,12 @@
 
 import os
 import bisect
-from os.path import join
 import csv
 import logging
 import numpy as np
 import scipy as sp
 
-from tral.paths import DATA_DIR
-SCORE_DIR = join(DATA_DIR, 'pvalue')
+from tral.paths import config_file, CONFIG_DIR
 
 LOG = logging.getLogger(__name__)
 
@@ -79,10 +77,20 @@ def empirical_list(l, n, sequence_type='AA', score_type='phylo'):
             else:
                 n -= 1
 
-    file = join(SCORE_DIR, sequence_type, score_type,
-                str(l) + '_' + str(n) + '.npz')
-    if not os.path.isfile(file):
-        raise ValueError("complete pdf file %s does not exist!" % file)
+    try:
+        file = config_file("data",
+                           "pvalue",
+                           sequence_type,
+                           score_type,
+                           str(l) + '_' + str(n) + '.npz')
+    except ValueError:
+        filename = os.path.join(CONFIG_DIR,
+                                "data",
+                                "pvalue",
+                                sequence_type,
+                                score_type,
+                                str(l) + '_' + str(n) + '.npz')
+        raise ValueError("Complete pdf file %s to calculate the pvalue does not exist!" % filename)
 
     empirical_list_all = np.load(file)
     # It is necessary to close the numpy filehandle.
@@ -168,7 +176,8 @@ def column_pdf(n, score_type='psim', sequence_type='AA'):
 
     # Currently, the pdfs are only save up to n_max = 150
     n_max = 150
-    with open(join(SCORE_DIR, sequence_type, score_type, str(min(n, n_max)) + '.txt'), 'r') as pdf_file:
+    filename = config_file("data", "pvalue", sequence_type, score_type, str(min(n, n_max)) + '.txt')
+    with open(filename, 'r') as pdf_file:
         pdf = [i for i in csv.reader(pdf_file, dialect='excel-tab')]
         pdf = [np.double(i[0]) for i in pdf[1:]]
         return np.array(pdf)
@@ -250,7 +259,7 @@ def calc_pvalues(repeats, result_file_path, file_name,
                     i_score_type)
             test_statistic = \
                 [pvalue_from_empirical_list(i_repeat, i_score_type,
-                                            empirical=empirical_list_local) \
+                                            empirical=empirical_list_local)
                  if i_repeat.l_effective != 0 else 1 for i_repeat in repeats]
         score_path = os.path.join(result_file_path, i_score_type)
         if not os.path.isdir(score_path):
@@ -327,8 +336,8 @@ def d_average_multiple_max_multinom(tandemrepeat, precision=10000.):
 
     p = d_average_multinom(l_repeat_structure[0], n_repeat_structure[0],
                            tandemrepeat.sequence_type, score_type='psim')
-    val = np.array((np.r_[l_repeat_structure[0]:(l_repeat_structure[0] * n_repeat_structure[0] + 1.)] * \
-          (precision / n_repeat_structure[0])).round(), dtype='uint32')
+    val = np.array((np.r_[l_repeat_structure[0]:(l_repeat_structure[0] * n_repeat_structure[0] + 1.)] *
+                    (precision / n_repeat_structure[0])).round(), dtype='uint32')
     if len(l_repeat_structure) == 1:
         # return best values first. for pSim, best = 1, worst = 0
         return p[::-1], val[::-1] / (precision * sum(l_repeat_structure))
@@ -340,8 +349,8 @@ def d_average_multiple_max_multinom(tandemrepeat, precision=10000.):
                                                score_type='psim')).ravel()
             val = (val[:, np.newaxis] +
                    np.array(
-                (np.r_[l_repeat_structure[i]:(l_repeat_structure[i] * n_repeat_structure[i] + 1.)]
-                 * (precision / n_repeat_structure[i])
+                (np.r_[l_repeat_structure[i]:(l_repeat_structure[i] * n_repeat_structure[i] + 1.)] *
+                 (precision / n_repeat_structure[i])
                  ).round(),
                 dtype='uint32')
             ).ravel()
@@ -427,8 +436,8 @@ def d_average_multiple_pars_multinom(tandemrepeat, precision=10000.):
         tandemrepeat.sequence_type, score_type='parsimony'
     )
     val = np.array(
-        (np.r_[(l_repeat_structure[0] * (n_repeat_structure[0] - 1)):-1:-1.]
-         * (precision / (n_repeat_structure[0] - 1.))
+        (np.r_[(l_repeat_structure[0] * (n_repeat_structure[0] - 1)):-1:-1.] *
+         (precision / (n_repeat_structure[0] - 1.))
          ).round(),
         dtype='uint32'
     )
@@ -446,8 +455,8 @@ def d_average_multiple_pars_multinom(tandemrepeat, precision=10000.):
                              ).ravel()
 
                 val = (val[:, np.newaxis] + np.array(
-                    (np.r_[(l_repeat_structure[i] * (n_repeat_structure[i] - 1)):-1:-1.]
-                     * (precision / (n_repeat_structure[i] - 1.))
+                    (np.r_[(l_repeat_structure[i] * (n_repeat_structure[i] - 1)):-1:-1.] *
+                     (precision / (n_repeat_structure[i] - 1.))
                      ).round(),
                     dtype='uint32')
                 ).ravel()
