@@ -9,18 +9,17 @@
 
 from collections import OrderedDict
 import distutils
-import itertools
 import logging
 import os
 import re
-import resource
 import shutil
 import subprocess
 import sys
 import tempfile
 
-from tral import configuration
-from tral.sequence import repeat_detection_io
+from .. import configuration
+from . import repeat_detection_io
+from ..paths import CONFIG_DIR, config_file
 
 LOG = logging.getLogger(__name__)
 
@@ -57,9 +56,9 @@ class BinaryExecutable:
             self.binary = distutils.spawn.find_executable(binary)
         if not self.binary:
             raise ValueError(
-                "The executable {} does not exist, although {} was selected "
-                "to be executed. Please make sure the executable is in the system path, or "
-                "the path to the executable is correctly set in config.ini".format(
+                "The executable {} does not exist, although {} was selected \n"
+                "to be executed. Please make sure the executable is in the system path, or \n"
+                "the path to the executable is set correctly in config.ini".format(
                     binary, name))
 
     def get_execute_tokens(self, *args):
@@ -103,8 +102,11 @@ def check_java_errors(outfile, errfile, log=LOG, procname=None):
     if errfile_size != 0 and outfile_size == 0:
         has_error = True
         LOG.warning(
-            "Process \"%s\" has empty STDOUT but non-empty STDERR",
-            procname)
+            "Process {} has empty STDOUT but non-empty STDERR.\n"
+            "This error often happens when {} is not installed or\n"
+            "no correct path is given in configfile .tral/config.ini.\n"
+            "Please check.".format(
+                procname, procname))
 
     pat_javaexc = re.compile(
         r'Exception in thread ".+" \S+:.*$(\s+at .+)+',
@@ -195,10 +197,22 @@ class DetectorHHrepID(TRDetector):
                 "-shuffle": False
             }
 
+            dummyhmm = os.path.expanduser(REPEAT_DETECTOR_PATH['HHrepID_dummyhmm'])
+            if not os.path.exists(dummyhmm):
+                # Special case for the data directory
+                try:
+                    rel_config = os.path.relpath(dummyhmm, start=CONFIG_DIR)
+                    if not rel_config.startswith(".."):
+                        dummyhmm = config_file(rel_config)
+                except FileNotFoundError:
+                    pass  # not in data; error below
+            if not os.path.exists(dummyhmm):
+                raise FileNotFoundError("HHrepID_dummyhmm not found: %s" % dummyhmm)
+
             self.valopts = {
                 # <file> input query alignment  (fasta/a2m/a3m) or HMM file (.hhm)
                 "-i": None,
-                "-d": REPEAT_DETECTOR_PATH['HHrepID_dummyhmm'],   # <path> dummy hmm database file
+                "-d": dummyhmm,   # <path> dummy hmm database file
                 "-o": 'hhrepID.o',    # <file> write results and multiple sequence alignment to file (default=none)
                 "-v": 0,           # -v: verbose mode (default: show only warnings)  ;  -v 0: suppress all screen outpu
                 "-P": None,        # <float> max p-value of suboptimal alignments in all search rounds but the last one (def=0.1)
@@ -298,7 +312,7 @@ class DetectorPhobos(TRDetector):
                 # <int> Display of TR. 0: asIs, 1: Alphabetical normal form, 2: Alphabetical normal form also considering the reverse complement. Default: 2
                 "--reportUnit": None,
                 "--NPerfectionMode": None,  # <int> Treatment of N.  0: asMismatch, 1: asNeutral, 2: asMatch. Default: 0.
-                "--printRepeatSeqMode": '3',  # <int> Show the repeat unit alignment
+                "--printRepeatSeqMode": '2',  # <int> Show the repeat unit alignment
                 "--outputFormat": None,  # <int> 0 : Use the Phobos output format. Default: 0
                 "--minPerfection": None,  # <float> Minimum perfection of a satellites. Default: 0.
                 "--maxPerfection": None,  # <float> Maximum perfection of a satellites. Default: 100.
@@ -556,10 +570,10 @@ class DetectorTRF(TRDetector):
                 # no redundancy elimination # Already without the -r flag set,
                 # the three best redundant solutions are displayed
                 "-r": False,
-                #"-h"            : False,    # When -h is set, the output html of interest is not written :(
-                #"-d"            : False,    # produce data file, not needed at curren
-                #"-m"            : False,    # masked sequence file
-                #"-f"            : False,    # flanking sequence
+                # "-h"            : False,    # When -h is set, the output html of interest is not written :(
+                # "-d"            : False,    # produce data file, not needed at curren
+                # "-m"            : False,    # masked sequence file
+                # "-f"            : False,    # flanking sequence
             }
 
             # The order of TRF flags matters. Hence, we use an ordered
@@ -653,7 +667,7 @@ class DetectorTrust(TRDetector):
             }
 
             self.valopts = {
-                "-matrix": REPEAT_DETECTOR_PATH['TRUST_substitutionmatrix'],
+                "-matrix": os.path.expanduser(REPEAT_DETECTOR_PATH['TRUST_substitutionmatrix']),
                 "-gapo": "8",
                 "-gapx": "2",
                 "-procTotal": "1",
@@ -720,22 +734,22 @@ class DetectorXStream(TRDetector):
 
         def __init__(self):
             self.boolopts = {
-                #"-n" : False,   # sequence type: nucleotide
+                # "-n" : False,   # sequence type: nucleotide
                 "-l": False,   # don't look for periods >1000
-                #"-p" : False,   # don't print repeat alignmen
+                # "-p" : False,   # don't print repeat alignmen
                 "-h": True,    # write to console, not HTML
-                #"-C" : False,   # don't print any repeat info
+                # "-C" : False,   # don't print any repeat info
                 "-z": True,  # generate spreadsheet .xls output
-                #"-Z" : False,   # create statistics spreadshee
-                #"-s" : False,   # parse output by input sequence
+                # "-Z" : False,   # create statistics spreadshee
+                # "-s" : False,   # parse output by input sequence
                 "-f": False,   # perform divide & conquer on inpu
-                #"-G" : False,   # create color-coded TR block diagram
-                #"-G*" : False,  # create single color TR block diagram
-                #"-B" : False,   # create sequence comparison map PNG
-                #"-O" : False,   # print "mismatch/gaps colored" HTML outpu
+                # "-G" : False,   # create color-coded TR block diagram
+                # "-G*" : False,  # create single color TR block diagram
+                # "-B" : False,   # create sequence comparison map PNG
+                # "-O" : False,   # print "mismatch/gaps colored" HTML outpu
                 "-N": False,   # turn off nesting
                 "-o": False,   # don't remove overlapping TR domain
-                #"-S" : False,   # input strain into database
+                # "-S" : False,   # input strain into database
                 "-V": True,   # don't invoke D & C automatically for long inpu
             }
 
@@ -749,11 +763,11 @@ class DetectorXStream(TRDetector):
                 "-e": None,    # [any number>=2] set minimum copy number
                 # "-a" : None,    # [string] add identifier string to output file
                 "-f": None,    # [integer] set fragment length
-                #"-d" : None,    # [file path] specify output directory
-                #"-A" : None,    # [file] import substitution alphabe
+                # "-d" : None,    # [file path] specify output directory
+                # "-A" : None,    # [file] import substitution alphabe
                 "-L": 6,    # [integer] set minimum TR domain length
                 "-P": 0,    # [0-1] set minimum % sequence coverage
-                #"-Q" : None,    # [user,pass,db] send output to MySQL TR database
+                # "-Q" : None,    # [user,pass,db] send output to MySQL TR database
             }
 
         def tokens(self, infile=None):
@@ -793,7 +807,7 @@ class DetectorXStream(TRDetector):
         files = os.listdir(searchdir)
 
         # look for the first file that matches XSTREAM_something_chart.xl
-        pat_chartfilename = re.compile("^XSTREAM_.*_chart\.xls$")
+        pat_chartfilename = re.compile(r"^XSTREAM_.*_chart\.xls$")
         for f in files:
             if pat_chartfilename.match(f):
                 return os.path.join(searchdir, f)
@@ -858,20 +872,18 @@ def Detectors(lDetector=None, sequence_type=None):
         sequence_type = CONFIG_GENERAL["sequence_type"]
     if not lDetector:
         lDetector = CONFIG_GENERAL["sequence"]["repeat_detection"][sequence_type]
-    if not isinstance(lDetector, list):
-        raise TypeError(""" lDetector is not of type list. Please supply a list of TR detectors
-        (e.g. lDetector = ['HHrepID']). If you use TR detectors defined in config.ini, make
-        sure the TR detector name is followed by a comma. E.g.: HHrepID,""")
     else:
-        if any(
-            i not in list(
-                itertools.chain(
-                *
-                CONFIG_GENERAL["sequence"]["repeat_detection"].values())) for i in lDetector):
+        if isinstance(lDetector, str):
+            lDetector = [lDetector]
+        elif not isinstance(lDetector, list):
+            raise TypeError(""" lDetector is not of type list. Please supply a list of TR detectors
+            (e.g. lDetector = ['HHrepID']). If you use TR detectors defined in config.ini, make
+            sure the TR detector list (AA or DNA) is correctly defined.""")
+        if any(i not in FINDER_LIST.keys() for i in lDetector):
             raise Exception(
                 "Unknown TR detector supplied (Supplied: {}. Known TR detectors: {})".format(
                     lDetector,
-                    FINDER_LIST))
+                    FINDER_LIST.keys()))
 
     DETECTORS = {FINDER_LIST[i].name: FINDER_LIST[i]() for i in lDetector}
 
